@@ -45,7 +45,6 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE         #
 # POSSIBILITY OF SUCH DAMAGE.                                             #
 # #########################################################################
-
 """
 Module for data correction and masking functions.
 """
@@ -54,7 +53,7 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 import numpy as np
-from scipy.ndimage import filters
+import scipy.ndimage
 import tomopy.util.mproc as mproc
 import tomopy.util.dtype as dtype
 import tomopy.util.extern as extern
@@ -66,25 +65,28 @@ from scipy.signal import medfilt2d
 
 logger = logging.getLogger(__name__)
 
-
-__author__ = "Doga Gursoy"
+__author__ = "Doga Gursoy, William Judge"
 __credits__ = "Mark Rivers, Xianghui Xiao"
 __copyright__ = "Copyright (c) 2015, UChicago Argonne, LLC."
 __docformat__ = 'restructuredtext en'
-__all__ = ['adjust_range',
-           'circ_mask',
-           'gaussian_filter',
-           'median_filter',
-           'median_filter_cuda',
-           'sobel_filter',
-           'remove_nan',
-           'remove_neg',
-           'remove_outlier',
-           'remove_outlier1d',
-           'remove_outlier_cuda',
-           'remove_ring',
-           'enhance_projs_aps_1id',
-           ]
+__all__ = [
+    'adjust_range',
+    'circ_mask',
+    'gaussian_filter',
+    'median_filter',
+    'median_filter_cuda',
+    'median_filter_nonfinite',
+    'median_filter3d',
+    'remove_outlier3d',
+    'sobel_filter',
+    'remove_nan',
+    'remove_neg',
+    'remove_outlier',
+    'remove_outlier1d',
+    'remove_outlier_cuda',
+    'remove_ring',
+    'enhance_projs_aps_1id',
+]
 
 
 def adjust_range(arr, dmin=None, dmax=None):
@@ -151,11 +153,14 @@ def gaussian_filter(arr, sigma=3, order=0, axis=0, ncore=None):
         ncore = mproc.mp.cpu_count()
 
     with cf.ThreadPoolExecutor(ncore) as e:
-        slc = [slice(None)]*arr.ndim
+        slc = [slice(None)] * arr.ndim
         for i in range(arr.shape[axis]):
             slc[axis] = i
-            e.submit(filters.gaussian_filter, arr[tuple(slc)], sigma,
-                     order=order, output=out[tuple(slc)])
+            e.submit(scipy.ndimage.gaussian_filter,
+                     arr[tuple(slc)],
+                     sigma,
+                     order=order,
+                     output=out[tuple(slc)])
     return out
 
 
@@ -186,10 +191,12 @@ def median_filter(arr, size=3, axis=0, ncore=None):
         ncore = mproc.mp.cpu_count()
 
     with cf.ThreadPoolExecutor(ncore) as e:
-        slc = [slice(None)]*arr.ndim
+        slc = [slice(None)] * arr.ndim
         for i in range(arr.shape[axis]):
             slc[axis] = i
-            e.submit(filters.median_filter, arr[tuple(slc)], size=(size, size),
+            e.submit(scipy.ndimage.median_filter,
+                     arr[tuple(slc)],
+                     size=(size, size),
                      output=out[tuple(slc)])
     return out
 
@@ -227,25 +234,25 @@ def median_filter_cuda(arr, size=3, axis=0):
 
         winAllow = range(2, 16)
 
-        if(axis != 0):
+        if (axis != 0):
             arr = np.swapaxes(arr, 0, axis)
 
         if size in winAllow:
-            loffset = int(size/2)
-            roffset = int((size-1)/2)
+            loffset = int(size / 2)
+            roffset = int((size - 1) / 2)
             prjsize = arr.shape[0]
             imsizex = arr.shape[2]
             imsizey = arr.shape[1]
 
             filter = tomocuda.mFilter(imsizex, imsizey, prjsize, size)
-            out = np.zeros(
-                shape=(prjsize, imsizey, imsizex), dtype=np.float32)
+            out = np.zeros(shape=(prjsize, imsizey, imsizex), dtype=np.float32)
 
             for step in range(prjsize):
                 # im_noisecu = arr[:][step][:].astype(np.float32)
                 im_noisecu = arr[step].astype(np.float32)
                 im_noisecu = np.lib.pad(im_noisecu, ((loffset, roffset),
-                                                     (loffset, roffset)), 'symmetric')
+                                                     (loffset, roffset)),
+                                        'symmetric')
                 im_noisecu = im_noisecu.flatten()
 
                 filter.setCuImage(im_noisecu)
@@ -254,7 +261,7 @@ def median_filter_cuda(arr, size=3, axis=0):
                 results = results.reshape(imsizey, imsizex)
                 out[step] = results
 
-            if(axis != 0):
+            if (axis != 0):
                 out = np.swapaxes(out, 0, axis)
         else:
             warnings.warn("Window size not support, using cpu median filter")
@@ -267,6 +274,195 @@ def median_filter_cuda(arr, size=3, axis=0):
     return out
 
 
+def median_filter_nonfinite(arr, size=3, callback=None):
+    """
+    Remove nonfinite values from a 3D array using an in-place 2D median filter.
+
+    The 2D selective median filter is applied along the last two axes of
+    the array.
+
+    .. versionadded:: 1.11
+
+    Parameters
+    ----------
+    arr : ndarray
+        The 3D array with nonfinite values in it.
+    size : int, optional
+        The size of the filter.
+    callback : func(total, description, unit)
+        A function called after every internal loop iteration.
+        total is number of loop iterations.
+        description is 'Nonfinite median filter'.
+        unit is ' prjs'.
+
+    Returns
+    -------
+    ndarray
+        The corrected 3D array with all nonfinite values removed based upon the
+        local median value defined by the kernel size.
+
+    Raises
+    ------
+    ValueError
+        If the filter comes across a kernel only containing non-finite values a
+        ValueError is raised for the user to increase their kernel size.
+
+    """
+    # Defining a callback function if None is provided
+    if callback is None:
+
+        def callback(total, description, unit):
+            pass
+
+    # Iterating throug each projection to save on RAM
+    for projection in arr:
+        nonfinite_idx = np.nonzero(~np.isfinite(projection))
+        projection_copy = projection.copy()
+
+        # Iterating through each bad value and replace it with finite median
+        for x_idx, y_idx in zip(*nonfinite_idx):
+
+            # Determining the lower and upper bounds for kernel
+            x_lower = max(0, x_idx - (size // 2))
+            x_higher = min(arr.shape[1], x_idx + (size // 2) + 1)
+            y_lower = max(0, y_idx - (size // 2))
+            y_higher = min(arr.shape[2], y_idx + (size // 2) + 1)
+
+            # Extracting kernel data and fining finite median
+            kernel_cropped_arr = projection_copy[x_lower:x_higher,
+                                                 y_lower:y_higher]
+
+            if len(kernel_cropped_arr[np.isfinite(kernel_cropped_arr)]) == 0:
+                raise ValueError(
+                    "Found kernel containing only non-finite values.\
+                                 Please increase kernel size")
+
+            median_corrected_arr = np.median(
+                kernel_cropped_arr[np.isfinite(kernel_cropped_arr)])
+
+            # Replacing bad data with finite median
+            projection[x_idx, y_idx] = median_corrected_arr
+
+        callback(arr.shape[0], 'Nonfinite median filter', ' prjs')
+
+    return arr
+
+
+def median_filter3d(arr, size=3, ncore=None):
+    """
+    Apply 3D median filter to a 3D array.
+
+    .. versionadded:: 1.13
+
+    Parameters
+    ----------
+    arr : ndarray
+        Input 3D array either float32 or uint16 data type.
+    size : int, optional
+        The size of the filter's kernel.
+    ncore : int, optional
+        Number of cores that will be assigned to jobs. All cores will be used
+        if unspecified.
+
+    Returns
+    -------
+    ndarray
+        Median filtered 3D array either float32 or uint16 data type.
+
+    Raises
+    ------
+    ValueError
+        If the input array is not three dimensional.
+
+    """
+    if ncore is None:
+        ncore = mproc.mp.cpu_count()
+
+    input_type = arr.dtype
+    if (input_type != 'float32') and (input_type != 'uint16'):
+        arr = dtype.as_float32(arr)  # silent convertion to float32 data type
+    out = np.empty_like(arr)
+    dif = 0.0  # set to 0 to avoid selective filtering
+
+    # convert the full kernel size (odd int) to a half size as the C function requires it
+    kernel_half_size = (max(int(size), 3) - 1) // 2
+
+    if arr.ndim == 3:
+        dz, dy, dx = arr.shape
+        if (dz == 0) or (dy == 0) or (dx == 0):
+            raise ValueError("The length of one of dimensions is equal to zero")
+    else:
+        raise ValueError("The input array must be a 3D array")
+
+    # perform full 3D filtering
+    if (input_type == 'float32'):
+        extern.c_median_filt3d_float32(np.ascontiguousarray(arr), out, kernel_half_size, dif, ncore,
+                                    dx, dy, dz)
+    else:
+        extern.c_median_filt3d_uint16(np.ascontiguousarray(arr), out, kernel_half_size, dif, ncore,
+                                    dx, dy, dz)
+    return out
+
+
+def remove_outlier3d(arr, dif, size=3, ncore=None):
+    """
+    Selectively applies 3D median filter to a 3D array to remove outliers. Also
+    called a dezinger.
+
+    .. versionadded:: 1.13
+
+    Parameters
+    ----------
+    arr : ndarray
+        Input 3D array either float32 or uint16 data type.
+    dif : float
+        Expected difference value between outlier value and the median value of
+        the array.
+    size : int, optional
+        The size of the filter's kernel.
+    ncore : int, optional
+        Number of cores that will be assigned to jobs. All cores will be used
+        if unspecified.
+
+    Returns
+    -------
+    ndarray
+        Dezingered 3D array either float32 or uint16 data type.
+
+    Raises
+    ------
+    ValueError
+        If the input array is not three dimensional.
+
+    """
+    if ncore is None:
+        ncore = mproc.mp.cpu_count()
+
+    input_type = arr.dtype
+    if (input_type != 'float32') and (input_type != 'uint16'):
+        arr = dtype.as_float32(arr)  # silent convertion to float32 data type
+    out = np.empty_like(arr)
+
+    # convert the full kernel size (odd int) to a half size as the C function requires it
+    kernel_half_size = (max(int(size), 3) - 1) // 2
+
+    if arr.ndim == 3:
+        dz, dy, dx = arr.shape
+        if (dz == 0) or (dy == 0) or (dx == 0):
+            raise ValueError("The length of one of dimensions is equal to zero")
+    else:
+        raise ValueError("The input array must be a 3D array")
+
+    # perform full 3D filtering
+    if (input_type == 'float32'):
+        extern.c_median_filt3d_float32(np.ascontiguousarray(arr), out, kernel_half_size, dif, ncore,
+                                    dx, dy, dz)
+    else:
+        extern.c_median_filt3d_uint16(np.ascontiguousarray(arr), out, kernel_half_size, dif, ncore,
+                                    dx, dy, dz)
+    return out
+
+
 def sobel_filter(arr, axis=0, ncore=None):
     """
     Apply Sobel filter to 3D array along specified axis.
@@ -292,7 +488,7 @@ def sobel_filter(arr, axis=0, ncore=None):
         ncore = mproc.mp.cpu_count()
 
     with cf.ThreadPoolExecutor(ncore) as e:
-        slc = [slice(None)]*arr.ndim
+        slc = [slice(None)] * arr.ndim
         for i in range(arr.shape[axis]):
             slc[axis] = i
             e.submit(filters.sobel, arr[slc], output=out[slc])
@@ -384,14 +580,16 @@ def remove_outlier(arr, dif, size=3, axis=0, ncore=None, out=None):
 
     ncore, chnk_slices = mproc.get_ncore_slices(arr.shape[axis], ncore=ncore)
 
-    filt_size = [size]*arr.ndim
+    filt_size = [size] * arr.ndim
     filt_size[axis] = 1
 
     with cf.ThreadPoolExecutor(ncore) as e:
-        slc = [slice(None)]*arr.ndim
+        slc = [slice(None)] * arr.ndim
         for i in range(ncore):
             slc[axis] = chnk_slices[i]
-            e.submit(filters.median_filter, arr[tuple(slc)], size=filt_size,
+            e.submit(scipy.ndimage.median_filter,
+                     arr[tuple(slc)],
+                     size=filt_size,
                      output=tmp[tuple(slc)])
 
     arr = dtype.as_float32(arr)
@@ -439,17 +637,20 @@ def remove_outlier1d(arr, dif, size=3, axis=0, ncore=None, out=None):
     other_axes = [i for i in range(arr.ndim) if i != axis]
     largest = np.argmax([arr.shape[i] for i in other_axes])
     lar_axis = other_axes[largest]
-    ncore, chnk_slices = mproc.get_ncore_slices(
-        arr.shape[lar_axis], ncore=ncore)
-    filt_size = [1]*arr.ndim
+    ncore, chnk_slices = mproc.get_ncore_slices(arr.shape[lar_axis],
+                                                ncore=ncore)
+    filt_size = [1] * arr.ndim
     filt_size[axis] = size
 
     with cf.ThreadPoolExecutor(ncore) as e:
-        slc = [slice(None)]*arr.ndim
+        slc = [slice(None)] * arr.ndim
         for i in range(ncore):
             slc[lar_axis] = chnk_slices[i]
-            e.submit(filters.median_filter, arr[slc], size=filt_size,
-                     output=tmp[slc], mode='mirror')
+            e.submit(scipy.ndimage.median_filter,
+                     arr[slc],
+                     size=filt_size,
+                     output=tmp[slc],
+                     mode='mirror')
 
     with mproc.set_numexpr_threads(ncore):
         out = ne.evaluate('where(arr-tmp>=dif,tmp,arr)', out=out)
@@ -497,13 +698,13 @@ def remove_outlier_cuda(arr, dif, size=3, axis=0):
 
         winAllow = range(2, 16)
 
-        if(axis != 0):
+        if (axis != 0):
             arr = np.swapaxes(arr, 0, axis)
 
         if size in winAllow:
             prjsize = arr.shape[0]
-            loffset = int(size/2)
-            roffset = int((size-1)/2)
+            loffset = int(size / 2)
+            roffset = int((size - 1) / 2)
             imsizex = arr.shape[2]
             imsizey = arr.shape[1]
 
@@ -513,7 +714,8 @@ def remove_outlier_cuda(arr, dif, size=3, axis=0):
             for step in range(prjsize):
                 im_noisecu = arr[step].astype(np.float32)
                 im_noisecu = np.lib.pad(im_noisecu, ((loffset, roffset),
-                                                     (loffset, roffset)), 'symmetric')
+                                                     (loffset, roffset)),
+                                        'symmetric')
                 im_noisecu = im_noisecu.flatten()
 
                 filter.setCuImage(im_noisecu)
@@ -522,7 +724,7 @@ def remove_outlier_cuda(arr, dif, size=3, axis=0):
                 results = results.reshape(imsizey, imsizex)
                 out[step] = results
 
-            if(axis != 0):
+            if (axis != 0):
                 out = np.swapaxes(out, 0, axis)
         else:
             warnings.warn("Window size not support, using cpu outlier removal")
@@ -535,9 +737,18 @@ def remove_outlier_cuda(arr, dif, size=3, axis=0):
     return out
 
 
-def remove_ring(rec, center_x=None, center_y=None, thresh=300.0,
-                thresh_max=300.0, thresh_min=-100.0, theta_min=30,
-                rwidth=30, int_mode='WRAP', ncore=None, nchunk=None, out=None):
+def remove_ring(rec,
+                center_x=None,
+                center_y=None,
+                thresh=300.0,
+                thresh_max=300.0,
+                thresh_min=-100.0,
+                theta_min=30,
+                rwidth=30,
+                int_mode='WRAP',
+                ncore=None,
+                nchunk=None,
+                out=None):
     """
     Remove ring artifacts from images in the reconstructed domain.
     Descriptions of parameters need to be more clear for sure.
@@ -588,9 +799,9 @@ def remove_ring(rec, center_x=None, center_y=None, thresh=300.0,
     dz, dy, dx = rec.shape
 
     if center_x is None:
-        center_x = (dx - 1.0)/2.0
+        center_x = (dx - 1.0) / 2.0
     if center_y is None:
-        center_y = (dy - 1.0)/2.0
+        center_y = (dy - 1.0) / 2.0
 
     if int_mode.lower() == 'wrap':
         int_mode = 0
@@ -602,14 +813,14 @@ def remove_ring(rec, center_x=None, center_y=None, thresh=300.0,
     if not 0 <= theta_min < 180:
         raise ValueError("theta_min should be in the range [0 - 180)")
 
-    args = (center_x, center_y, dx, dy, dz, thresh_max, thresh_min,
-            thresh, theta_min, rwidth, int_mode)
+    args = (center_x, center_y, dx, dy, dz, thresh_max, thresh_min, thresh,
+            theta_min, rwidth, int_mode)
 
     axis_size = rec.shape[0]
     ncore, nchunk = mproc.get_ncore_nchunk(axis_size, ncore, nchunk)
     with cf.ThreadPoolExecutor(ncore) as e:
         for offset in range(0, axis_size, nchunk):
-            slc = np.s_[offset:offset+nchunk]
+            slc = np.s_[offset:offset + nchunk]
             e.submit(extern.c_remove_ring, out[slc], *args)
     return out
 
@@ -679,8 +890,8 @@ def enhance_projs_aps_1id(imgstack, median_ks=5, ncore=None):
     """
     Enhance the projection images with weak contrast collected at APS 1ID
 
-    This filter uses a median fileter (will be switched to enhanced recursive 
-    median fileter, ERMF, in the future) for denoising, and a histogram 
+    This filter uses a median fileter (will be switched to enhanced recursive
+    median fileter, ERMF, in the future) for denoising, and a histogram
     equalization for dynamic range adjustment to bring out the details.
 
     Parameters
@@ -697,24 +908,24 @@ def enhance_projs_aps_1id(imgstack, median_ks=5, ncore=None):
     ndarray
         3D enhanced image stacks.
     """
-    ncore = mproc.mp.cpu_count()-1 if ncore is None else ncore
+    ncore = mproc.mp.cpu_count() - 1 if ncore is None else ncore
 
     # need to use multiprocessing to speed up the process
     tmp = []
     with cf.ProcessPoolExecutor(ncore) as e:
         for n_img in range(imgstack.shape[0]):
-            tmp.append(e.submit(_enhance_img,
-                                imgstack[n_img, :, :],
-                                median_ks,
-                                )
-                       )
+            tmp.append(e.submit(
+                _enhance_img,
+                imgstack[n_img, :, :],
+                median_ks,
+            ))
 
     return np.stack([me.result() for me in tmp], axis=0)
 
 
 def _enhance_img(img, median_ks, normalized=True):
     """
-    Enhance the projection image from aps 1ID to counter its weak contrast 
+    Enhance the projection image from aps 1ID to counter its weak contrast
     nature
 
     Parameters
@@ -727,7 +938,7 @@ def _enhance_img(img, median_ks, normalized=True):
         specify whether the enhanced image is normalized between 0 and 1,
         default is True
 
-    Returns 
+    Returns
     -------
     ndarray
         enhanced projection image
@@ -735,7 +946,7 @@ def _enhance_img(img, median_ks, normalized=True):
     wgt = _calc_histequal_wgt(img)
     img = medfilt2d(img, kernel_size=median_ks).astype(np.float64)
     img = ne.evaluate('(img**2)*wgt', out=img)
-    return img/img.max() if normalized else img
+    return img / img.max() if normalized else img
 
 
 def _calc_histequal_wgt(img):
@@ -750,7 +961,7 @@ def _calc_histequal_wgt(img):
     Returns
     -------
     ndarray
-        histogram euqalization weights (0-1) in the same shape as original 
+        histogram euqalization weights (0-1) in the same shape as original
         image
     """
-    return (np.sort(img.flatten()).searchsorted(img) + 1)/np.prod(img.shape)
+    return (np.sort(img.flatten()).searchsorted(img) + 1) / np.prod(img.shape)
